/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019,2021  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#include <ctype.h>
#include <genvector/gds_char.h>
#include "action.h"
#include "script.h"
#include "fgwirc.h"

#include "script_act.h"

static fgw_obj_t *scriptobj;

static fgw_error_t act_load(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	char *id, *fn, *lang;

	ACTION_HELP((
		"load - load an user script",
		"Syntax: load <id> <filename> <language>",
		"",
		"id is an arbitrary unique identifier for the script, filename is the path of the script file and language is a fungw recognized scripting language (e.g. fawk or lua or tcl, etc)",
		NULL));


	ACT_CONVARG(1, FGW_STR, id = argv[1].val.str);
	ACT_CONVARG(2, FGW_STR, fn = argv[2].val.str);
	ACT_CONVARG(3, FGW_STR, lang = argv[3].val.str);

	return script_load(wstatus, id, fn, lang);
}

static fgw_error_t act_unload(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	char *id;

	ACTION_HELP((
		"unload - unload an user script",
		"Syntax: unload <id>",
		"",
		"id is the same id that was given on load.",
		NULL));

	ACT_CONVARG(1, FGW_STR, id = argv[1].val.str);

	return script_unload(wstatus, id);
}

static fgw_error_t act_scripts(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	htsp_entry_t *e;
	gds_t tmp;

	ACTION_HELP((
		"scripts - list scripts loaded to the status window (window 0)",
		"Syntax: scripts",
		NULL));

	gds_init(&tmp);
	for(e = htsp_first(&scripts); e != NULL; e = htsp_next(&scripts, e)) {
		script_t *sc = e->value;
		gds_truncate(&tmp, 0);
		gds_append_str(&tmp, sc->obj->name);
		gds_append_str(&tmp, ": ");
		gds_append_str(&tmp, sc->fn);
		gds_append_str(&tmp, " (");
		gds_append_str(&tmp, sc->lang);
		gds_append(&tmp, ')');
		win_append(wstatus, tmp.array);
	}
	gds_uninit(&tmp);
	return 0;
}

static fgw_error_t act_hook_arg(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	action_details_t *actd;
	int idx;

	ACTION_HELP((
		"hook_arg - scripting: return an argument passed to the script hook/function",
		"Syntax: hook_arg <idx>",
		"",
		"idx shall be a positive number smaller than argc. Returns the requested arg as string",
		NULL));

	GET_ACTD(actd);
	ACT_CONVARG(1, FGW_INT, idx = argv[1].val.nat_int);

	if ((idx < 0) || (idx >= actd->argc))
		return FGW_ERR_ARG_CONV;

	res->type = FGW_STR;
	res->val.cstr = actd->argv[idx];
	return 0;
}

static fgw_error_t act_hook_block(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	action_details_t *actd;

	ACTION_HELP((
		"hook_block - scripting: do not run further stages of hooks",
		"Syntax: hook_block",
		NULL));

	GET_ACTD(actd);
	actd->res = ACTION_HOOK_BLOCK;
	return 0;
}


static fgw_error_t act_hook_from_nick(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	action_details_t *actd;

	ACTION_HELP((
		"hook_from_nick - scripting: return origin nick in a script hook/function",
		"Syntax: hook_from_nick",
		"",
		"Returns a string with the nickname/servername or empty if there is no origin",
		NULL));

	GET_ACTD(actd);
	res->type = FGW_STR;
	res->val.cstr = actd->from->nick;
	return 0;
}

static fgw_error_t act_hook_from_ident(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	action_details_t *actd;

	ACTION_HELP((
		"hook_from_ident - scripting: return origin ident in a script hook/function",
		"Syntax: hook_from_ident",
		"",
		"Returns a string with the ident or empty if there is no origin or origin is not a nick",
		NULL));

	GET_ACTD(actd);
	res->type = FGW_STR;
	res->val.cstr = actd->from->ident;
	return 0;
}

static fgw_error_t act_hook_from_host(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	action_details_t *actd;

	ACTION_HELP((
		"hook_from_host - scripting: return origin hostname in a script hook/function",
		"Syntax: hook_from_host",
		"",
		"Returns a string with the host or empty if there is no origin or origin is not a nick",
		NULL));

	GET_ACTD(actd);
	res->type = FGW_STR;
	res->val.cstr = actd->from->host;
	return 0;
}


static fgw_error_t act_hook_from_hostmask(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	char tmp[1700];
	action_details_t *actd;

	ACTION_HELP((
		"hook_from_ident - scripting: return origin hostmask in a script hook/function",
		"Syntax: hook_from_hostmask",
		"",
		"Returns a string with the hostmask (nick!ident@hostname) or empty if there is no origin or origin is not a nick",
		NULL));

	GET_ACTD(actd);
	sprintf(tmp, "%s!%s@%s", actd->from->nick, actd->from->ident, actd->from->host);
	res->type = FGW_STR | FGW_DYN;
	res->val.str = fgw_strdup(tmp);
	return 0;
}

static fgw_error_t act_hook_server(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	action_details_t *actd;

	ACTION_HELP((
		"hook_server - scripting: return the server ID the irc message is comig from",
		"Syntax: hook_server",
		"",
		"Returns -1 if no server is available",
		NULL));

	GET_ACTD(actd);
	res->type = FGW_INT;
	res->val.nat_int = (actd->server == NULL) ? -1 : actd->server->sid;
	return 0;
}


static fgw_error_t act_script_help(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	res->type = FGW_INT;

	if (argc < 3) {
		ACTION_HELP((
			"script_help - scripting: mandatory action help text",
			"Syntax: script_help <firstarg> <help_line1> [help_line2] ...",
			"",
			"The first argument of the action has to be passed as firstarg.",
			"Returns non-zero if help triggered.",
			NULL));
		res->val.nat_int = 1;
		return 0;
	}

	res->val.nat_int = 0;

	if ((argc > 1) && ((argv[1].type & FGW_STR) == FGW_STR) && (argv[1].val.str[0] == ACTION_HELP_STR[0]) && (strcmp(argv[1].val.str, ACTION_HELP_STR) == 0)) {
		int n;
		win_t *curr = idx2win(win_curr_idx);

		win_append(curr, "");
		win_append(curr, "--- help begin ---");
		for(n = 2; n < argc; n++)
			ACT_CONVARG(n, FGW_STR, win_append(curr,argv[n].val.str));
		win_append(curr, "--- help end ---");
		win_append(curr, "");
		res->val.nat_int = 1;
	}

	return 0;
}

static fgw_error_t act_num_server_slots(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	ACTION_HELP((
		"num_server_slots - scripting: return the number of server slots allocated",
		"Syntax: num_server_slots",
		"",
		"Note: some slots may be empty. Slots are indexed from 0.",
		NULL));

	res->type = FGW_INT;
	res->val.nat_int = irc_servers.used;
	return 0;
}

static fgw_error_t act_strutil(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	const char *cmd, *s;
	
	ACTION_HELP((
		"strutil - scripting: various string utilities",
		"Syntax:",
		" strutil strpbrk <str> <chrs>  - scripting: strpbrk(3); returns offs or -1",
		" strutil tolower <str>         - scripting: retrun lowercaste str",
		NULL));

	ACT_CONVARG(1, FGW_STR, cmd = argv[1].val.str);

	if (strcmp(cmd, "strpbrk") == 0) {
		ACT_CONVARG(2, FGW_STR, ;);
		ACT_CONVARG(3, FGW_STR, ;);

		s = strpbrk(argv[2].val.str, argv[3].val.str);
		res->type = FGW_INT;
		res->val.nat_int = (s == NULL) ? -1 : s - argv[2].val.str;
	}
	else if (strcmp(cmd, "tolower") == 0) {
		char *s, *head;

		ACT_CONVARG(2, FGW_STR, head = fgw_strdup(argv[2].val.str));

		for(s = head; *s != '\0'; s++)
			*s = tolower(*s);

		res->type = FGW_STR | FGW_DYN;
		res->val.str = head;
	}
	else
		return -1;

	return 0;
}

void script_act_init(void)
{
	scriptobj = fgw_obj_reg(&fgw, "script");

	fgw_func_reg(scriptobj, "load", act_load);
	fgw_func_reg(scriptobj, "unload", act_unload);
	fgw_func_reg(scriptobj, "scripts", act_scripts);

	/* hook accessors */
	fgw_func_reg(scriptobj, "hook_arg", act_hook_arg);
	fgw_func_reg(scriptobj, "hook_block", act_hook_block);
	fgw_func_reg(scriptobj, "hook_from_nick", act_hook_from_nick);
	fgw_func_reg(scriptobj, "hook_from_ident", act_hook_from_ident);
	fgw_func_reg(scriptobj, "hook_from_host", act_hook_from_host);
	fgw_func_reg(scriptobj, "hook_from_hostmask", act_hook_from_hostmask);
	fgw_func_reg(scriptobj, "hook_server", act_hook_server);
	fgw_func_reg(scriptobj, "script_help", act_script_help);

	/* utility functions with return value */
	fgw_func_reg(scriptobj, "num_server_slots", act_num_server_slots);
	fgw_func_reg(scriptobj, "strutil", act_strutil);
}
